@@ -36,7 +36,7 @@
 
 #include "grl-data.h"
 #include "grl-log.h"
-#include <grl-registry.h>
+#include "grl-registry-priv.h"
 
 #define GRL_LOG_DOMAIN_DEFAULT data_log_domain
 GRL_LOG_DOMAIN(data_log_domain);
@@ -48,14 +48,11 @@ struct _GrlDataPrivate {
 static void grl_data_finalize (GObject *object);
 static void free_list_values (GrlKeyID key, GList *values, gpointer user_data);
 
-#define GRL_DATA_GET_PRIVATE(o)                                         \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GRL_TYPE_DATA, GrlDataPrivate))
-
 static void free_list_values (GrlKeyID key, GList *values, gpointer user_data);
 
 /* ================ GrlData GObject ================ */
 
-G_DEFINE_TYPE (GrlData, grl_data, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (GrlData, grl_data, G_TYPE_OBJECT);
 
 static void
 grl_data_class_init (GrlDataClass *klass)
@@ -63,14 +60,12 @@ grl_data_class_init (GrlDataClass *klass)
   GObjectClass *gobject_class = (GObjectClass *)klass;
 
   gobject_class->finalize = grl_data_finalize;
-
-  g_type_class_add_private (klass, sizeof (GrlDataPrivate));
 }
 
 static void
 grl_data_init (GrlData *self)
 {
-  self->priv = GRL_DATA_GET_PRIVATE (self);
+  self->priv = grl_data_get_instance_private (self);
   self->priv->data = g_hash_table_new_full (g_direct_hash,
                                             g_direct_equal,
                                             NULL,
@@ -616,6 +611,137 @@ grl_data_get_int64 (GrlData *data, GrlKeyID key)
   }
 }
 
+/**
+ * Returns whether the string is a canonical one.
+ **/
+static gboolean
+is_canonical (const gchar *key)
+{
+  if (key == NULL) {
+    return FALSE;
+  }
+
+  for (; *key != '\0'; key++) {
+    if (*key != '-' &&
+        (*key < '0' || *key > '9') &&
+        (*key < 'A' || *key > 'Z') &&
+        (*key < 'a' || *key > 'z'))
+      return FALSE;
+  }
+
+  return TRUE;
+}
+
+/**
+ * grl_data_set_for_id:
+ * @data: data to change
+ * @key_name: name of the key to change or add 
+ * @value: the new value
+ *
+ * Sets the first value associated with @key_name in @data. This @key_name is used to create
+ * a new #GParamSpec instance, which is further used to create and register a key using
+ * grl_registry_register_metadata_key(). If @key_name already has a first @value, old
+ * value is replaced by the new one.
+ *
+ * A property key_name consists of segments consisting of ASCII letters and
+ * digits, separated by either the '-' or '_' character. The first
+ * character of a property key_name must be a letter. Key_names which violate these
+ * rules lead to undefined behaviour.
+ *
+ * Returns: TRUE if @value was set to @key_name, FALSE otherwise.
+ *
+ * Since: 0.3.6
+ **/
+gboolean
+grl_data_set_for_id (GrlData *data, const gchar *key_name, const GValue *value)
+{
+  GrlRegistry *registry;
+  GrlKeyID key_id;
+
+  key_name = g_intern_string (key_name);
+  g_return_val_if_fail (is_canonical (key_name), FALSE);
+
+  registry = grl_registry_get_default ();
+  key_id = grl_registry_lookup_metadata_key (registry, key_name);
+
+  if (key_id != GRL_METADATA_KEY_INVALID) {
+    grl_data_set (data, key_id, value);
+    return TRUE;
+  }
+
+  GRL_DEBUG ("%s is not a registered metadata-key", key_name);
+  key_id = grl_registry_register_metadata_key_for_type(registry, key_name, G_VALUE_TYPE (value));
+  if (key_id != GRL_METADATA_KEY_INVALID)
+    grl_data_set (data, key_id, value);
+
+  return (key_id != GRL_METADATA_KEY_INVALID);
+}
+
+
+/**
+ * grl_data_add_for_id:
+ * @data: data to change
+ * @key_name: name of the key to change or add 
+ * @value: the new value
+ *
+ * Appends the value associated with @key_name to @data. This @key_name is used to create
+ * a new #GParamSpec instance, which is further used to create and register a key using
+ * grl_registry_register_metadata_key().
+ *
+ * A property key_name consists of segments consisting of ASCII letters and
+ * digits, separated by either the '-' or '_' character. The first
+ * character of a property key_name must be a letter. Key_names which violate these
+ * rules lead to undefined behaviour.
+ *
+ * Returns: TRUE if @value was added to @key_name, FALSE otherwise.
+ *
+ * Since: 0.3.6
+ **/
+gboolean
+grl_data_add_for_id (GrlData *data, const gchar *key_name, const GValue *value)
+{
+  GrlRegistry *registry;
+  GrlKeyID key_id;
+
+  key_name = g_intern_string (key_name);
+  g_return_val_if_fail (is_canonical (key_name), FALSE);
+
+  registry = grl_registry_get_default ();
+  key_id = grl_registry_lookup_metadata_key (registry, key_name);
+
+  if (key_id == GRL_METADATA_KEY_INVALID) {
+    GRL_DEBUG ("%s is not a registered metadata-key", key_name);
+    key_id = grl_registry_register_metadata_key_for_type (registry, key_name, G_VALUE_TYPE (value));
+    if (key_id == GRL_METADATA_KEY_INVALID) {
+      return FALSE;
+    }
+  }
+
+  switch (G_VALUE_TYPE (value)) {
+  case G_TYPE_INT:
+    grl_data_add_int (data, key_id, g_value_get_int (value));
+    break;
+
+  case G_TYPE_INT64:
+    grl_data_add_int64 (data, key_id, g_value_get_int64 (value));
+    break;
+
+  case G_TYPE_FLOAT:
+    grl_data_add_float (data, key_id, g_value_get_float (value));
+    break;
+
+  case G_TYPE_STRING:
+    grl_data_add_string (data, key_id, g_value_get_string (value));
+    break;
+
+  default:
+    GRL_WARNING ("'%s' is being ignored as G_TYPE is not being handled", key_name);
+    return FALSE;
+  }
+
+  return TRUE;
+}
+
 /**
  * grl_data_remove:
  * @data: data to change
