@@ -50,14 +50,9 @@ struct _GrlRelatedKeysPrivate {
 static void grl_related_keys_finalize (GObject *object);
 static void free_value (GValue *val);
 
-#define GRL_RELATED_KEYS_GET_PRIVATE(o)                                 \
-  (G_TYPE_INSTANCE_GET_PRIVATE ((o),                                    \
-                                GRL_TYPE_RELATED_KEYS,                  \
-                                GrlRelatedKeysPrivate))
-
 /* ================ GrlRelatedKeys GObject ================ */
 
-G_DEFINE_TYPE (GrlRelatedKeys, grl_related_keys, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (GrlRelatedKeys, grl_related_keys, G_TYPE_OBJECT);
 
 static void
 grl_related_keys_class_init (GrlRelatedKeysClass *klass)
@@ -65,14 +60,12 @@ grl_related_keys_class_init (GrlRelatedKeysClass *klass)
   GObjectClass *gobject_class = (GObjectClass *)klass;
 
   gobject_class->finalize = grl_related_keys_finalize;
-
-  g_type_class_add_private (klass, sizeof (GrlRelatedKeysPrivate));
 }
 
 static void
 grl_related_keys_init (GrlRelatedKeys *self)
 {
-  self->priv = GRL_RELATED_KEYS_GET_PRIVATE (self);
+  self->priv = grl_related_keys_get_instance_private (self);
   self->priv->data = g_hash_table_new_full (g_direct_hash,
                                             g_direct_equal,
                                             NULL,
