@@ -70,11 +70,6 @@ GRL_LOG_DOMAIN(registry_log_domain);
 #define SOURCE_IS_INVISIBLE(src)                                \
   GPOINTER_TO_INT(g_object_get_data(G_OBJECT(src), "invisible"))
 
-#define GRL_REGISTRY_GET_PRIVATE(object)                        \
-  (G_TYPE_INSTANCE_GET_PRIVATE((object),                        \
-                               GRL_TYPE_REGISTRY,               \
-                               GrlRegistryPrivate))
-
 /* GQuark-like implementation, where we manually assign the first IDs. */
 struct KeyIDHandler {
   GHashTable *string_to_id;
@@ -131,13 +126,11 @@ enum {
 };
 static gint registry_signals[SIG_LAST];
 
-G_DEFINE_TYPE (GrlRegistry, grl_registry, G_TYPE_OBJECT);
+G_DEFINE_TYPE_WITH_PRIVATE (GrlRegistry, grl_registry, G_TYPE_OBJECT);
 
 static void
 grl_registry_class_init (GrlRegistryClass *klass)
 {
-  g_type_class_add_private (klass, sizeof (GrlRegistryPrivate));
-
   /**
    * GrlRegistry::source-added:
    * @registry: the registry
@@ -291,7 +284,7 @@ network_changed_cb (GObject     *gobject,
 static void
 grl_registry_init (GrlRegistry *registry)
 {
-  registry->priv = GRL_REGISTRY_GET_PRIVATE (registry);
+  registry->priv = grl_registry_get_instance_private (registry);
 
   registry->priv->configs =
     g_hash_table_new_full (g_str_hash, g_str_equal, g_free, (GDestroyNotify) configs_free);
@@ -572,6 +565,74 @@ grl_registry_register_metadata_key_full (GrlRegistry *registry,
   return registered_key;
 }
 
+G_GNUC_INTERNAL GrlKeyID
+grl_registry_register_metadata_key_for_type (GrlRegistry *registry,
+                                             const gchar *key_name,
+                                             GType type)
+{
+  GParamSpec *spec;
+
+  switch (type) {
+  case G_TYPE_INT:
+    spec = g_param_spec_int (key_name,
+                             key_name,
+                             key_name,
+                             0, G_MAXINT,
+                             0,
+                             G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE);
+    break;
+
+  case G_TYPE_INT64:
+    spec = g_param_spec_int64 (key_name,
+                               key_name,
+                               key_name,
+                               -1, G_MAXINT64,
+                               -1,
+                               G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE);
+    break;
+
+  case G_TYPE_STRING:
+    spec = g_param_spec_string (key_name,
+                                key_name,
+                                key_name,
+                                NULL,
+                                G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE);
+    break;
+
+  case G_TYPE_BOOLEAN:
+    spec = g_param_spec_boolean (key_name,
+                                 key_name,
+                                 key_name,
+                                 FALSE,
+                                 G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE);
+    break;
+
+  case G_TYPE_FLOAT:
+    spec = g_param_spec_float (key_name,
+                               key_name,
+                               key_name,
+                               0, G_MAXFLOAT,
+                               0,
+                               G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE);
+    break;
+
+  default:
+    if (type == G_TYPE_DATE_TIME) {
+        spec = g_param_spec_boxed (key_name,
+                                   key_name,
+                                   key_name,
+                                   G_TYPE_DATE_TIME,
+                                   G_PARAM_STATIC_STRINGS | G_PARAM_READWRITE);
+    } else {
+      GRL_WARNING ("'%s' is being ignored as G_TYPE '%s' is not being handled",
+                   key_name, G_VALUE_TYPE_NAME (type));
+      return GRL_METADATA_KEY_INVALID;
+    }
+  }
+
+  return grl_registry_register_metadata_key (registry, spec, GRL_METADATA_KEY_INVALID, NULL);
+}
+
 static void
 key_id_handler_init (struct KeyIDHandler *handler)
 {
